// simple fragment shader

varying vec3 N;
varying vec3 V;
varying vec3 L;
vec3 H;

vec3 vecN, vecV, vecL, vecH;

uniform float roughness;
const float PI = 3.141592;

vec4 fresnel(float x) {
	vec4 specular = gl_FrontMaterial.specular;
	return specular + ( 1.0 - specular) * pow( (1.0 - x), 5.0) ;
}

vec4 cookTorrance() {
	float NdotL =  dot(vecN, vecL);
	float NdotV =  dot(vecN, vecV);
	float NdotH =  dot(vecN, vecH);
	float HdotV =  dot(vecH, vecV);
	float roughnessSquared = roughness * roughness;

	float senNandH = sqrt(1.0 - pow(NdotH, 2.0));
	float tangentOfAlpha = senNandH / NdotH;

	vec4 F = fresnel( HdotV );

	
	float G_a = 2.0 * NdotH * NdotV / (HdotV + 0.0001); // para não ocorrer divisão por zero...
	float G_b = 2.0 * NdotH * NdotL / (HdotV + 0.0001);
	float G = min( 1.0, min ( G_a, G_b ) );

	float notSoR = 1.0 / (( 4.0 * roughnessSquared * pow( NdotH, 4.0) )+ 0.0001 );
	float stillNotR_a = (NdotH * NdotH) - 1.0;
	float stillNotR_b = (roughnessSquared * NdotH * NdotH) + 0.0001;
	float stillNotR = exp( stillNotR_a / stillNotR_b );
	float R = stillNotR * notSoR;

	return gl_LightSource[0].specular  * max(0.0, NdotL) * ((F * G * R)/(( NdotV *  NdotL) + 0.00001));	
 
}

vec4 orenNayar() {
	float NdotL =  dot(vecN, vecL);
	float NdotV =  dot(vecN, vecV);
	float NdotH =  dot(vecN, vecH);
	float HdotV =  dot(vecH, vecV);
	float HdotL =  dot(vecH, vecL);
	
	float roughnessSquared = pow(roughness , 1.0);
	float alpha = max( acos(NdotV), acos(NdotL) );
	float beta = min( acos(NdotV), acos(NdotL) );
	float gama = dot(  normalize( vecV - (vecN * NdotV) )  ,  normalize( vecL - (vecN*NdotL) )  );

	float C1 = 1.0   -   (   0.5 * ( roughnessSquared / ( roughnessSquared + 1.0/3.0 ) )   );
	float C2 = 0.45 * ( roughnessSquared /( roughnessSquared + 0.09 ) );
	if( gama >= 0.0 ){
		C2 *= sin(alpha);
	} else {
		C2 *= ( sin(alpha) - pow(2.0*beta/PI, 3.0) );
	}	
	float C3 = (1.0/8.0) * ( roughnessSquared/( roughnessSquared + 0.09 ) ) * ( pow( (4.0*alpha*beta)/pow(PI, 2.0), 2.0) );

	float epsilon = ( 1.0 - abs(gama) )  *  C3  *  tan( (alpha+beta)/2.0 );

	float chi = gama * C2 * tan(beta);
	
	NdotL = max(0.0, NdotL);
	return gl_LightSource[0].diffuse * gl_FrontMaterial.diffuse *  NdotL * (C1 + chi + epsilon );
}

void main()
{
	vec3 H = V + L;

	vecN = normalize(N);
	vecV = normalize(V);
 	vecL = normalize(L);
	vecH = normalize(H);

	gl_FragColor =  orenNayar() + cookTorrance();
}
